#!/usr/local/bin/perl
#
# vote_ctl.cgi
#	簡易投票システム 制御用スクリプト -- CGI_Lite & Stand Alone版
#
# このスクリプトの使用方法
#
#	準備
#		1. CGI_Lite.pmのLite.pmを用意する。
#				（CPANからCGI_Lite-2.02.tar.gzをダウンロードして解凍すると，その中にLite.pmがある)
#		2. CGIを実行可能かつ書き込み可能なディレクトリに，次の三つのファイルを置く。（最初はこの三つ以外は置かない）
#				vote_ctl.cgi
#				vote.cgi
#				Lite.pm
#		3. 両ファイルの第１行目を実行可能なPerlインタプリタの絶対パスに変更する。
#		4. 両ファイルに実行の許可を与える。
#		5. vote_ctl.cgiにパスワードなどを設定し，出題者以外がアクセスできないようにする（必須ではない）。
#
#	問題ファイルの送信
#		1. 問題ファイルを作成。ファイルのフォーマットについてのインストラクションは，次の操作で表示される。
#		2. 出題制御に使用するパソコンからvote_ctl.cgiにアクセスする。
#		3. 問題ファイルを指定して [Setup] ボタンを押す。
#		4. 出題準備OK！が出れば準備完了。 [FirstQ] ボタンを押せば第１問が表示される。
#
#	出題・回答受付
#		1. 問題確認後，[StartAns] ボタンで回答受付開始。
#		2. “現在問題を受付中”と点滅表示されることを確認の上，回答を受け付ける。
#		3. 十分に回答を受け付けた後，[CloseAns] ボタンを押す。
#		4. “回答受付締切”と表示され，回答数分布が帯グラフとともに示される。
#
#	出題制御
#		1. [PrevQ] ボタンで前問に戻る。
#		2. [PresentQ] ボタンで同じ問題を再出題。
#		3. [NextQ] ボタンで次の問題に移る。
#		4. [InitQ] ボタンで問題ファイル送信画面に戻る。
#		5. [ResetQ] ボタンで最後に送信した問題ファイルを最初から出題し直す。
#
#
########################  Initiallizations  ########################
#
# Required libraries and modules
#
use strict;
use File::Path;
use CGI::Lite;		#システムにGCI::Lite.pmがインストール済みの時
#use Lite;			#システムにCGI::Lite.pmがインストールされていないか，またはインストールされているかどうかわからないとき
use Encode;
use utf8;
binmode STDOUT, ':encoding(utf8)';

my $Version = "0.51.01";							# バージョンナンバー

my $DebugInfo;									# デバッグ用変数（この変数にHTMLを代入しておくと，制御ボタンの下に情報が表示される）
my $Exist;										# 問題の有無をチェックする変数

# Directories and Files
my $MyPathName;
	if ($ENV{SCRIPT_FILENAME} ne "") { $MyPathName = $ENV{SCRIPT_FILENAME}; }				# スクリプトのパス付ファイル名
	else { die "Environmental variable is not available.\n"; }
my $MyName = $MyPathName;
my @MyDir = ($MyName =~ m|(.*)/|g);				# スクリプトのパスを切り出す。gをつけてリストで受けると最後の“/”の前まで切り出される。
my $MyDir = @MyDir[0];
$MyName =~ s|.*/||;								# フルパスからファイル名のみを切り出す。
chdir "$MyDir";									# スクリプトのあるディレクトリで動作させる。

my $AnswerDir = "$MyDir/Answer";				# 回答書込みディレクトリ
	unless (-d $AnswerDir) {
		mkdir $AnswerDir, 0777 or die $!;
		chmod 0777, $AnswerDir or die $!;
	}
	
my $AnswerFileName = "Answer";					# 回答記録ファイルのファイル名
my $AnswerFile = "$AnswerDir/$AnswerFileName";	# 回答記録ファイルのパス付ファイル名
my $QFile = "$AnswerDir/Questions.txt";			# 問題ファイル名（Setup時にアップロードされた問題ファイルをこの名前でセーブ）
my $QNumberFile = "$AnswerDir/QNumber";			# 問題番号制御用ファイル
	unless (-e $QNumberFile) {
		open QN, ">:encoding(sjis)", $QNumberFile or die "Failed in opening $QNumberFile in line ".__LINE__." : $!";
			print QN "0";
		close QN;
		chmod 0664, $QNumberFile;
	}
	
my $VoteFormName = "vote_form.html";			# 回答送信用フォームのファイル名
my $VoteForm = "$MyDir/$VoteFormName";			#      同     上               （パス付）
my $VoteEndName = "vote_end.html";				# 回答送信後に表示されるファイル名
my $VoteEnd = "$MyDir/$VoteEndName";			#      同     上                 （パス付）
	
# Question Control
my $QNumber;									# 問題番号制御変数
my $Question;									# 問題
my @Ans;										# 選択肢リスト用変数
my @Question;									# 問題と選択肢リストをまとめた変数

# CGI::Liteオブジェクトの初期化
my $Cgi = new CGI::Lite();						# CGIオブジェクト変数初期化
$Cgi -> add_timestamp(0);
$Cgi -> set_platform ('Unix');
$Cgi -> set_directory($AnswerDir) or die $!;
my %Data = $Cgi -> parse_form_data();

# 色の設定
my $color1="#8844FF";
my $color2="#008800";
my $color3="#000080";
my $color4="#cc00cc";
my @Color = ("white", "red", "blue", "green", "maroon", "orange", "olive", "magenta", "yellow", "black");

# 全角-半角変換する文字列
my $zen = "－　＿／［］＠．Ａ-Ｚａ-ｚ０-９";
my $han = "- _/[]@.A-Za-z0-9";

########################  Main Routine  ########################

# 問題ファイルの送信画面を返す
if    ( $Data{Button} eq "" || $Data{Button} eq "InitVote") { &SendHTML("RequestQFile"); }

# 新しい問題の受信および初期設定
elsif ( $Data{Button} eq "Setup" || $Data{Button} eq "ResetQ" ) {
	if ( $Data{Button} eq "Setup" ) { &SaveQFile; }
	if ( -e $AnswerFile ) { unlink $AnswerFile; }
	&ResetQNumber;
	&SendHTML("Ready");
}

# 次（最初）の出題
elsif ( $Data{Button} eq "NextQ" || $Data{Button} eq "FirstQ" ) {
	$QNumber = &GetNextQNumber;
	($Exist, @Question) = &GetQuestion($QFile, $QNumber);		# 次(最初)の問題の問題文と解答リストを得る
	$QNumber = &GetPresentQNumber;
	($Exist, @Question) = &GetQuestion($QFile, $QNumber);
	&CreateVoteForm(@Question);
	&CreateVoteEnd;
	&SendHTML("SetQ", @Question);
}

# 前問を再出題
elsif ($Data{Button} eq "PrevQ" ) {	
	$QNumber = &GetPrevQNumber;
	if ($QNumber <= 0) { $QNumber = &SetQNumber(1); }
	($Exist, @Question) = &GetQuestion($QFile, $QNumber);
	&CreateVoteForm(@Question);
	&SendHTML("SetQ", @Question);
}

# 現在の問題を再出題
elsif ($Data{Button} eq "PresentQ" ) {	
	$QNumber = &GetPresentQNumber;
	($Exist, @Question) = &GetQuestion($QFile, $QNumber);
	&SendHTML("SetQ", @Question);
}

# 回答受付準備
elsif ( $Data{Button} eq "StartAns" ) {
	$QNumber = &GetPresentQNumber;
	($Exist, @Question) = &GetQuestion($QFile, $QNumber);
	&SetAnswerStatus("Accept", @Question);
	&SendHTML("WaitAns", @Question);
}

# 回答受付終了，回答分析
elsif ( $Data{Button} eq "CloseAns" ) {
	$QNumber = &GetPresentQNumber;
	($Exist, @Question) = &GetQuestion($QFile, $QNumber);
	my $PresentAnswerFile = &SetAnswerStatus("Reject");
	my @Answer = &AnalAns($PresentAnswerFile);
	&SendHTML("StopAns", @Answer);
}

else {
	&SendHTML("RequestQFile");
}

########################  Subroutines  ########################

################## Q & A Control #####################
#########
# AnalAns 
#    回答分析  各選択肢への回答の頻度分布(百分率)を返す
sub AnalAns {
	my $AnsFile = shift;
	my @PDist = &GetAnswerDist($AnsFile);
	return (@PDist);
}

#########
# ReadAnswer
#    回答ファイルを読み出し，回答部分の生リスト返す。
sub ReadAnswer {
	my $AnsFile = shift;
	open ANS, "<:encoding(sjis)", $AnsFile or die "vote_ctl.cgi: Failed in opening $AnsFile in line ".__LINE__." : $!";
		my @AnsData = <ANS>;
	close ANS;
	shift @AnsData;
	while (my $q = shift @AnsData) {
		chomp;
		last if ( $q =~ m/^[\-#]+$/ );
	}
	foreach (@AnsData) {
		chomp;
		s/^\s+//;
	}
	return @AnsData;
}

#########
# GetAnswerDist 
#    回答を分析し，各選択肢への頻度分布を百分率で返す
sub GetAnswerDist {
	
	my $AnsFile = shift;
	my @Dist;
	my @PDist;
	my $NumNoAns = 0;
	my $NumValidAns = 0;
	my $i;
	
	my @Ans = &ReadAnswer($AnsFile);
	my $TotalNumAns = $#Ans + 1;
	
	for ($i=0; $i<=$#Ans; $i++) {
		
		chomp $Ans[$i];
		
		if ($Ans[$i] =~ /\d+/) {
			$Dist[$Ans[$i]]++;
			$NumValidAns++;
		}
		else {
			$NumNoAns++;
		}
	}
	
	for ($i=0; $i<=$#Dist; $i++) {
		$PDist[$i] = int ($Dist[$i] / $NumValidAns * 100 + 0.5);
	}
	
	return ($TotalNumAns, $NumValidAns, $NumNoAns, @PDist);
	
}

#########
# SetAnswerStatus 
#    回答待ち受け状態をスイッチする”。
#        回答待ち開始時には"Answer"ファイルを用意
#        回答待ち終了時には"Answer"ファイルを"Answer_###"にリネーム
sub SetAnswerStatus {
	
	my $Ctr = lc(shift);
	my $NewAnswerFile;
	my $GlobAnswerFiles = $AnswerFileName . "*";
	my $NextNumber = "000";
	
	if ( $Ctr eq "accept" ) {
		
		if (-e $AnswerFile) {
			chdir $AnswerDir;
			my @AnsFiles = glob($GlobAnswerFiles);
			@AnsFiles = reverse sort @AnsFiles;
			for (my $i=0; $i<=$#AnsFiles; $i++) {
				if ($AnsFiles[$i] =~ m/Answer_(\d\d\d)/) {
					$NextNumber = sprintf "%03d", ($1 + 1);
					last;
				}
			}
			$NewAnswerFile = $AnswerFile . "_" . $NextNumber;
			rename $AnswerFile, $NewAnswerFile;
		}
		
		my $Q = "[Q]";
		$Q .= shift;
		my @A = @_;
		my $AStr;
		for (my $i=1; $i<=$#A+1; $i++) {
			$AStr .= "<$i> $A[$i-1]\n";
		}
		
		open ANS, ">:encoding(sjis)", $AnswerFile or die "Failed in opening $AnswerFile in line ".__LINE__." : $!";
			print ANS "----------\n";
			print ANS "$Q\n$AStr";
			print ANS "----------\n";
		close ANS;
		
		chmod 0664, $AnswerFile;
		
	} elsif ($Ctr eq "reject") {
		
		if (-e $AnswerFile) {
			chdir $AnswerDir;
			my @AnsFiles = glob($GlobAnswerFiles);
			@AnsFiles = reverse sort @AnsFiles;
			for (my $i=0; $i<=$#AnsFiles; $i++) {
				if ($AnsFiles[$i] =~ m/Answer_(\d\d\d)/) {
					$NextNumber = sprintf "%03d", ($1 + 1);
					last;
				}
			}
			$NewAnswerFile = $AnswerFile . "_" . $NextNumber;
			rename $AnswerFile, $NewAnswerFile;
		}
		
		return $NewAnswerFile;
		
	}
}

#########
# SaveQFile()
sub SaveQFile {
	my $SentFile = $AnswerDir . "/". $Data{QFile};
	rename $SentFile, $QFile or die $!;
	chmod 0664, $QFile;
}

#########
# ResetQNumber()
sub ResetQNumber {
	if ( -e $QNumberFile ) { unlink $QNumberFile; }
}

#########
# GetPrevQNumber()
sub GetPrevQNumber { return GetQNumber(-1); }

#########
# GetPresentQNumber()
sub GetPresentQNumber { return GetQNumber(0); }

#########
# GetNextQNumber()
sub GetNextQNumber { return GetQNumber(1); }

#########
# GetQNumber()
#   Returns the question number to be readed.
#   Argument = -1;        returns previous number
#   Argument = 0 or none; returns present number
#   Argument = 1 ;        returns next number
sub GetQNumber {
	my $Act;
	if ($_[0] eq "") { $Act = 1; }
	else { $Act = $_[0]; }
	
	open QN, "<:encoding(sjis)", $QNumberFile or die "Failed in opening $QNumberFile in line ".__LINE__." : $!";
		my $QN = <QN>;
	close QN;
	
	$QN += $Act;
	
	open QN, ">:encoding(sjis)", $QNumberFile or die "Failed in opening $QNumberFile in line ".__LINE__." : $!";
		print QN $QN;
	close QN;
	
	return $QN;
}

#########
# SetQNumber()
#   Sets QN as argued.
sub SetQNumber {
	
	my $QN = shift;
	
	open QN, ">:encoding(sjis)", $QNumberFile or die "Failed in opening $QNumberFile in line ".__LINE__." : $!";
		print QN $QN;
	close QN;
	
	return $QN;
}

#########
# GetQuestion()
#   Returns a list consist of a question and the answer options.
#   Returning List = ( ExistFlag, Qusetion, option 1, option 2, ...)
sub GetQuestion {
	(my $QF, my $QN) = @_;
	my $NextQN = $QN + 1;
	my $Q;
	my @A;
	my $Exist = 0;
	
	open QF, "<:encoding(sjis)", $QF or die "Failed in opening $QF in line ".__LINE__." : $!";
		while (<QF>) {
			chomp;
			if ( /^\[$QN\](.*)/ ) {
				$Exist = 1;
				$Q .= $1;
				last;
			}
		}
		while (<QF>) { 
			chomp;
			last if ( /^\[$NextQN\].*/ );
			if (/^<(\d+)>/) { 
				$A[$1-1] = $';				#'
				$A[$1-1] =~ s/^\s+//;
				$A[$1-1] =~ s/\s+$//;
				next;
			}
			$Q .= $_;
		}
	close QF;
	$Q =~ s/^\s+//;
	$Q =~ s/\s+$//;
	$Q =~ s/[\r\n]//g;

	if ($Exist==0) {
		SetQNumber($QN-1);
		($Exist, $Q, @A) = &GetQuestion($QF, $QN-1);
		
	}

	return ($Exist, $Q, @A);
}

################## Sending HTML #####################

#########
# GetHTMLBody() : function returning HTML body strings depending on the situation

sub GetHTMLBody {
	
	my $MesType = shift;

	my $HTML_Body = "\t<form method='post' action='$MyName' enctype='multipart/form-data'>\n\n";
		
	# 問題ファイル，初期設定ファイルの送信
	if ($MesType eq "RequestQFile") {
		$HTML_Body .= <<"__REQUESTQFILE_1__";
		
		<p style='color:$color3; font-weight:bold'>問題のリストを送信してください。</p>
		<div>
			<p style='font-size:1.0em'>ファイルの形式は次の形式のテキストファイルにしてください。<br />
			<p style='margin-left:50pt; font-size:1.0em'>
				[1]<span style='color:$color4; font-size:0.8em'>{改行}</span><br />
				問題文１はここに書きます。<span style='color:$color4; font-size:0.8em'>{改行}</span><br />
				<1>ここに選択肢の1<span style='color:$color4; font-size:0.8em'>{改行}</span><br />
				<2>ここに選択肢の2<span style='color:$color4; font-size:0.8em'>{改行}</span><br />
				<3>ここに選択肢の3<span style='color:$color4; font-size:0.8em'>{改行}</span><br />
				<4>今のところ選択肢はいくつでもかまいません。<span style='color:$color4; font-size:0.8em'>{改行}</span><br />
				<span style='color:$color4; font-size:0.8em'>{改行}</span> <span style="font-size:0.8em; color:green"><- あってもなくても可</span><br />
				[2]<span style='color:$color4; font-size:0.8em'>{改行}</span><br />
				問題文2はここに書きます。<span style='color:$color4; font-size:0.8em'>{改行}</span><br />
				問題文は複数行に亘ってもかまいません。<span style='color:$color4; font-size:0.8em'>{改行}</span><br />
				<1>各選択肢は1行におさめてください。<span style='color:$color4; font-size:0.8em'>{改行}</span><br />
				<2>選択肢は2行以上にはしないでください。<span style='color:$color4; font-size:0.8em'>{改行}</span><br />
				<3>選択肢を2行にしたら，どのように動作するか未確認です。<span style='color:$color4; font-size:0.8em'>{改行}</span><br />
				<4>選択肢の数は問題ごとに変えてもかまいません。<span style='color:$color4; font-size:0.8em'>{改行}</span><br />
				<5>回答フォームに選択肢数の変更が反映されないときは，リロードしてもらってください。<span style='color:$color4; font-size:0.8em'>{改行}</span><br />
				<6>ネットワークのキャッシュの加減で回答フォームがすぐには更新されない場合もありえます。その場合はどうしようもありません。<span style='color:$color4; font-size:0.8em'>{改行}</span><br />
			</p>
		</div>
		<p class='cap'><span style='color:$color1; font-weight:bold'>問題文リストファイル</span>:<br />
		<input type='file' name='QFile' size='50'></p>
__REQUESTQFILE_1__

		$HTML_Body .= &GetHTMLButton("Setup", "ResetQ");
		
	}
	
	# 問題受領完了
	elsif ($MesType eq "Ready") {
		
		$HTML_Body .= "<p class='cap'>出題準備OK！</p>\n";
		$HTML_Body .= &GetHTMLButton ("FirstQ");
		
	}
	
	# 問題提示
	elsif ($MesType eq "SetQ") {
		
		my $Q = shift;
		my @A = @_;
		
		$HTML_Body .= "<p class='question'>[Q $QNumber]<br />$Q\n";
		$HTML_Body .= "<ol class='options'>\n";
		for (my $i=0; $i<=$#A; $i++) {
			$HTML_Body .= "<li>$A[$i]</li>\n";
		}
		$HTML_Body .= "</ol></p>\n";
		
		$HTML_Body .= &GetHTMLButton("StartAns", "PrevQ", "PresentQ", "NextQ");
	}
	
	# 回答待ち
	elsif ($MesType eq "WaitAns") {
		
		my $Q = shift;
		my @A = @_;
		
		$HTML_Body .= "<p class='question'>[Q $QNumber]<br />$Q\n";
		$HTML_Body .= "<ol class='options'>\n";
		for (my $i=0; $i<=$#A; $i++) {
			$HTML_Body .= "<li>$A[$i]</li>\n";
		}
		$HTML_Body .= "</ol></p>\n";
		
#“現在回答を受付中”を点滅させるときは次の行をアンコメント。IEでは動作しない。
#		$HTML_Body .= "<p class='cap' style='color:red; text-align:center; font-size:2.0em; text-decoration:blink'>";
#“現在回答を受付中”を横スクロールさせるときは次の行をアンコメント。速度などを調整可
		$HTML_Body .= "<p class='cap' style='color:red; text-align:center; font-size:2.0em'><marquee scrollamount='2' scrolldelay='5'>";
		$HTML_Body .= "現在回答を受付中";
		$HTML_Body .= "</marquee></p>\n";		# 点滅させたときは</marquee>はゴミになるが，気にしてない。
		$HTML_Body .= &GetHTMLButton("CloseAns", "PrevQ", "PresentQ", "NextQ");
	}
	
	# 解答収集終了
	elsif ($MesType eq "StopAns") {
		
		my $NumTotalAns = shift;
		my $NumValidAns = shift;
		my $NumNoAns = shift;
		my @PAnswer = @_;
		
		my $i;
		my $j;
		my $k;
		my ($Exist, @QInfo) = &GetQuestion($QFile, $QNumber);
		my $Q = shift @QInfo;
		my @A = @QInfo;
		
		$HTML_Body .= "<p class='question'>[Q $QNumber]<br />$Q\n";
		$HTML_Body .= "<ol class='options'>\n";
		for (my $i=0; $i<=$#A; $i++) {
			$HTML_Body .= "<li>$A[$i]</li>\n";
		}
		$HTML_Body .= "</ol></p>\n";
		
		$HTML_Body .= "<p class='cap' style='color:blue; text-align:center; font-size:2.0em'><u>回答受付締切</u></p>\n";
		
		$HTML_Body .= "<p class='cap' style='color:black; margin-top:30pt; text-align:center'>";
		for ($i=1; $i<=$#PAnswer; $i++) {
			$HTML_Body .= "<span style='color:" . $Color[$i] . "'>回答" . $i . "</span>&nbsp;:&nbsp;" . $PAnswer[$i] . "% &nbsp;&nbsp;\n"
		}
		
		$HTML_Body .= "</p>\n<p style='text-align:center; font-size:0.5em; line-height:0.7; letter-spacing:-0.3em'>\n";
		
		for ($k=0; $k<15; $k++) {
			for ($i=1; $i <= $#PAnswer; $i++) {
				$HTML_Body .= "<span style='color:" . $Color[$i] . "'>";
				for ($j=0; $j<$PAnswer[$i]; $j++) { $HTML_Body .= "■";	}
				$HTML_Body .= "</span>";
			}
			$HTML_Body .= "<br />";
		}
		
		$HTML_Body .= "</p>\n";
		
		$HTML_Body .= "<p class='cap' style='text-align:right; margin-top:20pt;'>";
		$HTML_Body .= "<span style='color:blue'>総回答数</span>：$NumTotalAns；&nbsp;&nbsp;";
		$HTML_Body .= "<span style='color:Blue'>有効回答</span>：$NumValidAns；&nbsp;&nbsp;";
		$HTML_Body .= "<span style='color:Blue'>無回答</span>：$NumNoAns";
		$HTML_Body .= "</p>\n";
		
		$HTML_Body .= &GetHTMLButton("StartAns", "PrevQ", "PresentQ", "NextQ");
		
	}
	# エラーメッセージ
	elsif ($MesType eq "Error") {
		
		(my $ErCode, my $ErType, my $ErAdvice, my $ErVal, my $ErMess, my $ErAdvice, my $ButtonVal) = @_;
		
		$HTML_Body .= "<p><span style='color:$color1; font-weight:bold'>";
		$HTML_Body .= "エラー：$ErCode</p><p>";
		$HTML_Body .= $ErMess;
		$HTML_Body .= "</p><p><p><pstyle='color:$color4'>";
		$HTML_Body .= "$ErAdvice";
		$HTML_Body .= "</p><p><p>";
		$HTML_Body .= "<form method='post' action='$MyPathName'>";
		
		$HTML_Body .= &GetHTMLButton($ButtonVal);
		
	}
	
	$HTML_Body .= "</form>\n";

	return $HTML_Body;
}

#########
# GetHTMLQuestion()
sub GetHTMLQuestion {
	my $Q = $_[0];
	$Q =~ tr/\n/<br \/>/;
	return $Q;
}

#########
# GetHTMLButton()
sub GetHTMLButton { 
	my $MaxButton = $#_;
	my $BtnStr = "<hr width='70%' align='center'>\n<p align='center'>\n";
	for (my $i = 0; $i <= $MaxButton; $i++) {
		$BtnStr .= "<input type='submit' name='Button' value='$_[$i]' style='font-size:1.2em;width:120px;margin-left:5px;'>\&nbsp\;\n";
	}
	$BtnStr .= "</p>\n";
	$BtnStr .= "<p style='text-align:right;margin-bottom:20px'><input type='submit' name='Button' value='InitQ'></p><hr>\n";

	return $BtnStr;
}

#########
# SendHTML()
sub SendHTML {
	
	my @Args = @_;
	
	&SendHTTPHeader(0);
	&SendHTMLTop;
	print &GetHTMLBody(@Args);
	&SendHTMLBtm;
	exit(0);
	
}	

#########
# SendHTTPHeader;
sub SendHTTPHeader {
	if ( $_[0] eq "" || $_[0] == 0 ) {
		print "Content-Type: text/html\n\n";
	} elsif ($_[0] == 1) {
		print "Location: 'http://" . $ENV{SERVER_NAME} . $_[1] . "\n\n";
		exit(1);
	}
}

#########
# SendHTMLTop()
sub SendHTMLTop {	
	print <<"__TOPofHTML__";
<html>
<head>
	<meta http-equiv="Content-Language" content="ja">
	<meta http-equiv="Content-Type" content="text/html; charset=utf-8">
	<title>Web Vote Controller</title>
	<style type="text/css">
	<!--
		body {
			font-size: 20px;
		}
		p {	margin-top:10pt;
			margin-bottom:10pt;
			font-size:20px; }
		li {	margin-top:5pt; }
		.question {	font-size:1.5em;
					color:$color4; }
		.options {	font-size:1.5em;
					color:$color3; }
		.cap { font-size:1.5em; }
	--> 
	</style>
</head>
<body bgcolor='white'>

	<hr />
	
__TOPofHTML__
}

#########
# SendHTMLBtm()
sub SendHTMLBtm {	
	
	my $DebugInfoPart;
	
	unless ($DebugInfo eq "") {
		$DebugInfo =~ s|\n|<br />|g;	#/
		$DebugInfoPart = "<p style='text-align:left;color:$color4'><b>Debugging Informations:</b><br />$DebugInfo</p>";	
	}
	
	print <<"__BOTTOMofHTML__";
</p>

		<h3 style='color:$color2; text-align:right; font-family:times; font-style:italic'>
			VOTE on the WEB
			<span style="font-size:0.5em;color:$color4">ver. $Version</span>
		</h3>

$DebugInfoPart
</body>
</html>

__BOTTOMofHTML__

}

################## Creating HTML Files #####################

#########
# CreateVoteForm 
#    回答送信用フォームファイルの作成
sub CreateVoteForm {
	
	my $Q = shift;
	my @Ans = @_;
	my $NumAns = $#Ans;
	my $i;
	
	open VF, ">:encoding(utf-8)", $VoteForm or die "Failed in opening $VoteForm in line " . __LINE__ . " : $!";
	
	print VF <<"__VOTEFORM_1__";
<html>
<head>
<meta http-equiv="Content-Language" content="ja">
<meta http-equiv="Content-Type" content="text/html; charset=utf-8">
<title>Sending Answer</title>
</head>
<body>
<hr>
<h1 style="color:#000080;text-align:center">回答送信フォーム</h1>
<h2 style="color:#000080;text-align:center">Answer Form</h2>
<hr>
<p style="text-align:center">回答番号を選んで送信してください。<br />Select your answer and press 'Send'.</p>
<p style="text-align:center;font-size:0.8em;color:blue">回答受付中以外は送信できません。<br />Your answer will be received only when accepting.</p>
<form METHOD="POST" action="vote.cgi" name="Answer" enctype="multipart/form-data">
<p style="text-align:center">
<input type="radio" name="Ans" value="N" selected="selected">NoAnswer<br />
__VOTEFORM_1__
				
	for ($i=1; $i<=$NumAns+1; $i++) {
		print VF "<label><input type='radio' name='Ans' value='$i'>&nbsp;&nbsp; $i</label><br />\n";
	}
	
	print VF <<"__VOTEFORM_2__";
</p>
<p style="text-align:center">
<input type="submit" name="Submit" value="Send">
</p>
</form>

<hr>

<p>
<a href="vote.cgi">回答送信フォームのリロード</a>
</p>

<p></p><hr><p></p><p></p><p></p>

</body>
</html>
__VOTEFORM_2__

	close VF;
	
	chmod 0664, $VoteForm;

}

#########
# CreateVoteEnd 
#    送信完了メッセージの作成
sub CreateVoteEnd {
	open VE, ">:encoding(utf-8)", $VoteEnd or die "Failed in opening $VoteEnd in line ".__LINE__." : $!";
	
		print VE <<__VOTEEND__;
<html>
<head>
<meta http-equiv="Content-Language" content="ja">
<meta http-equiv="Content-Type" content="text/html; charset=utf-8">
<title>Answer Received</title>
</head>
<body>
<hr>
<h1 style="color:#000080;text-align:center">回答終了</h1>
<h2 style="color:#000080;text-align:center">Answer Received</h2>
<hr style="margin-bottom:20pt">
<div align="center">
<p>回答を受け取りました。</p>
<p><a href="vote.cgi">回答送信フォームに戻る</a></p>
</div>
</body>
</html>
__VOTEEND__

	close VE;
}

################## Error Handling #####################
#########
# Error handling routine.
sub Error {

	my $ErrorCode=$_[0];
	my $ErrorType;
	my $ErrorAdvice; 	
	my $ErrorVal;
	my $ErrorMessage;
	my $ErrorAdvice;
	my $ButtonVal;
	my $key;
	my %in;

	# フォームが送られてこなかった.
	if ($ErrorCode == 0) {
		$ButtonVal = "";
		$ErrorType = "";
		$ErrorMessage .= "フォームが送られていません。";
		$ErrorAdvice  .= "やり直してください。";
	}
	# 
	elsif ($ErrorCode == 1) {
		&SendHTTPHeader(1);
		exit (1);
	}
	
	my @ErrorData = ($ErrorCode,
					  $ErrorType,
					  $ErrorAdvice, 
					  $ErrorVal,
					  $ErrorMessage,
					  $ErrorAdvice,
					  $ButtonVal);
		
	&SendHTML("Error", @ErrorData)

}


